/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.chainofresponsibility;

import dk.rode.thesis.flyweight.Character;
import dk.rode.thesis.flyweight.Whitespace;
import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.log.LogFactory;

/**
  * A <i>whitespace handler</i> is a handler that can handle
  * {@link Whitespace whitespace} characters.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
@Participant("ConcreteHandler")
public class WhitespaceHandler extends CharacterHandler {

    /**
      * No-arg constructor. <p>
      * 
      * This handler is not registered to any 
      * handler {@link HandlerChain chain} until specifically 
      * registered using the {@link #register(HandlerChain)} 
      * method.
      */
    public WhitespaceHandler() {
        super();
    }
    
    /**
      * Constructor, which registers this symbol handler to the
      * handler chain supplied as {@code chain}.
      * 
      * @param  chain The handler chain; cannot be null.
      * @throws NullPointerException If {@code chain} is null.
      */
    public WhitespaceHandler(HandlerChain<Character> chain) {
        super(chain);
    }
    
    /**
      * 
      */
    protected boolean handle(Character character) {
        if (character instanceof Whitespace) {
            LogFactory.getLog().println("Whitespace handled!");
            return true;
        } 
        return false;
    }
    
}
