/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */
package dk.rode.thesis.decorator;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>duplex sequence decorator</i> returns the String value
  * returned by the decorated sequence concatenated with
  * the same value, for example {@code x -> xx}. <p>
  * 
  * Only sequences returning strings can be decorated. <p>
  * 
  * The default stringable policy used is {@link DecoratedPolicy#HIDDEN}.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
@Participant("ConcreteDecorator")
public class DuplexDecorator extends SequenceDecorator<String> implements Sequence<String> {

    /**
      * Constructor.
      * 
      * @param  sequence The sequence to decorate; cannot be null.
      * @throws NullPointerException If {@code sequence} is null.
      */
    public DuplexDecorator(@Participant("ConcreteComponent") Sequence<String> sequence) {
        super(sequence, DecoratedPolicy.HIDDEN);
    }

    /**
      * 
      */
    public DuplexDecorator copy() {
        return new DuplexDecorator(this.getSequence(true));
    }

    /**
      * 
      */
    public String current() {
        String s = super.current();
        return s + s;
    }
    
    /**
      * 
      */
    public String next() {
        String s = super.next();
        return s + s;
    }
    
}
