/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Decorator</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.175]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Attach additional responsibilities to an object
  *    dynamically. Decorators provide a flexible alternative to 
  *    subclassing for extending functionality.
  * 
  * </i></ul>
  * 
  * Here, the <b>Component</b> participant is represented by the
  * {@link dk.rode.thesis.meta.model.Sequence} interface. 
  * Any {@code Sequence} implementation thus represent the 
  * <b>ConcreteComponent</b> participant. <p>
  * 
  * The <b>Decorator</b> participant is represented by the abstract
  * {@link dk.rode.thesis.decorator.SequenceDecorator} class,
  * which also implements the {@code Sequence} interface as required.
  * It implements all the basic {@code Sequence} functionality by
  * forwarding calls to the decorated sequence. This package defines
  * several decorators that represent the <b>ConcreteDecorator</b>
  * participant, for example {@link dk.rode.thesis.decorator.AppenderDecorator}
  * and {@link dk.rode.thesis.decorator.DuplexDecorator}. <p>
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<p>
  * <img src="doc-files/Decorator.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:<br>
  * All the decorators defined here only adds behaviour, no additional state.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Decorator", 
         scope        = Scope.Object,
         purpose      = Purpose.Structural,
         participants = {"Component", "ConcreteComponent", "Decorator", "ConcreteDecorator"})
package dk.rode.thesis.decorator;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;
