/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.interpreter;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.log.LogFactory;

/**
  * An <i>equal expression</i> can determine if the result of 
  * evaluating two expressions is equal or not, regardless
  * of types.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    CompareExpression
  * @see    NotExpression
  * @see    AndExpression
  * @see    OrExpression
  */
@Participant("NonTerminalExpression")
public class EqualExpression 
     extends BinaryExpression<Object,Boolean> 
  implements NonTerminalExpression<Boolean> {
    
    /**
      * Constructor.
      * 
      * @param  first  The first expression operand; cannot be null.
      * @param  second The second expression operand; cannot be null.
      * @throws NullPointerException If either expression is null.
      */
    public EqualExpression(Expression<?> first, Expression<?> second) {
        super(first, second);
    }
    
    /**
      * Copy constructor.
      * 
      * @param  expression The expression to copy; cannot be null.
      * @throws NullPointerException If {@code expression} is null.
      */
    public EqualExpression(EqualExpression expression) {
        super(expression);
    }

    /**
      * 
      */
    public EqualExpression copy() {
        return new EqualExpression(this);
    }
    
    /**
      * @return {@code equal}
      */
    public final String name() {
        return "equal";
    }
    
    /**
      * 
      */
    public Boolean evaluate(Context context) throws ExpressionException {
        LogFactory.getLog(this).println("Evaluating: ", this.asSymbol(context.reset()));
        return this.first.evaluate(context).equals(this.second.evaluate(context));
    }
    
    /**
      * Returns {@code x == y}, where {@code x} is the symbolic 
      * representation of the first expression operand and {@code y} 
      * of the the second.  
      */
    public String asSymbol(Context context) throws ExpressionException {
        return this.asSymbol(context, "=="); 
    }
    
}
