/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */
package dk.rode.thesis.meta;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.LOCAL_VARIABLE;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
  * Identifies a <b>participant</b> as described in [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.7]: a class
  * or object participating in the design pattern. <p>
  * 
  * The scope of the design pattern is considered to be the (parent) package
  * enclosing the identified participant. The annotated target represents the 
  * participant. For example, if the target is a field, the object stored 
  * in the field is the participant. <p>
  *  
  * If the annotated target is a type, the participant is declared by
  * the type.  
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    ParticipantUsage
  */
@Documented
@Inherited
@Retention(RUNTIME)
@Target({TYPE, METHOD, PARAMETER, FIELD, LOCAL_VARIABLE})
public @interface Participant {
    
    /**
      * The roles this participant fulfil in the pattern.
      */
    String[] value();
            
}
