/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */
package dk.rode.thesis.meta.model;

/**
  * A <i>long sequence</i> represents an unbounded long value that 
  * is increased with each call to {@link #next()}. <p>
  * 
  * A long sequence is {@link Sequence#bounded() unbounded}, but
  * {@link #unique() unique} and {@link Sequence#consistent() consistent}.
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public class LongSequence extends AbstractSequence<Long> implements Sequence<Long> {

    /**
      * Start number of this sequence.  
      */
    private final long start;
    /**
      * The current sequence number.  
      */
    private long sequence;
    
    /**
      * No-arg constructor.
      */
    public LongSequence() {
        this(0);
    }

    /**
      * Constructor.
      * 
      * @param start The start value of this sequence.
      */
    public LongSequence(long start) {
        super();
        this.sequence = this.start = start;
    }

    /**
      * Copy constructor.
      * 
      * @param  sequence The sequence to copy; cannot be null.
      * @throws NullPointerException If {@code sequence} is null.
      */
    public LongSequence(LongSequence sequence) {
        super(sequence);
        this.start = sequence.start;
        this.sequence = sequence.sequence;
    }
    
    /**
      * 
      */
    public LongSequence copy() {
        return new LongSequence(this); 
    }
    
    /**
      * Returns false.
      *  
      * @return False. 
      */
    public boolean bounded() {
        return false;
    }

    /**
      * Returns true.
      *  
      * @return True.
      */
    public boolean consistent() {
        return true;
    }

    /**
      * Returns true.
      *  
      * @return True.
      */
    public boolean unique() {
        return true;
    }
    
    /**
      * 
      */
    public Long current() {
        return this.sequence;
    }

    /**
      * 
      */
    public Long next() {
        if (this.state != State.NORMAL) {
            this.state = State.NORMAL;
        }
        return ++this.sequence;
    }

    /**
      * 
      */
    public void reset() {
        super.reset();
        this.sequence = this.start;
    }

}
