/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.meta.reflect;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
  * A method or constructor annotated with the <i>executor</i>
  * annotation indicates that it will be executed reflectively
  * by a given {@link #value() context}. <p>
  *  
  * <b>Implementation notes</b>:<br>
  * The executor annotation can annotate any constructor or method.
  * The context using the annotated members must supply the logic
  * to support the parameter types. <p>
  * 
  * The annotation is inherited! This means that if defined in
  * a super-class, a sub-class overriding the annotated member
  * can be used as an observer if an instance of the sub-class
  * is used. Inheritance is only on from super-class to sub-class,
  * not for implemented interfaces.  
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    ExecutorProcessor
  */
@Inherited
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.CONSTRUCTOR, ElementType.METHOD})
public @interface Executor {

    /**
      * Returns the <i>context</i> to which the annotated 
      * member is associated. <p>
      * 
      * The context is used to discriminate between the
      * contexts using the annotated members, for example
      * different context annotating members in the same
      * class for different purposes. 
      *  
      * @return The context; default {@code Object} (any).
      */
    Class<?> value() default Object.class;
    
}
