/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.meta.test;

import dk.rode.thesis.meta.log.Log;
import dk.rode.thesis.meta.util.Arguments;

/**
  * A <i>testable</i> class is used to perform <i>overall
  * functional tests</i>, reporting the results to a 
  * {@link Log log} using the {@link #test(Log, Arguments)} method, 
  * but is <b>NOT</b> meant as a replacement for unit testing,
  * for example using JUnit. The tests performed by implementations
  * of this interface normally have a higher level or granularity
  * compared to unit testing. <p>
  * 
  * In case the tests succeeds, true is returned from the method;
  * in case the tests fails, false is returned. The method may also
  * throw an exception, in which case the tests are concluded to
  * have failed as well.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public interface Testable {

    /**
      * Performs the specific tests and report the outcome to the
      * {@link Log log} supplied as {@code log}. <p>
      * 
      * Arguments to control log-level: <ul><p>
      * 
      * <li> {@code -log}: a boolean value (as a string) indicating if 
      *      class logging is enabled for relevant classes. <p> 
      * 
      * <li> {@code -log.verbose}: a boolean value (as a string) indicating
      *      if class logging is enabled for relevant classes that generates
      *      verbose output. <p>  
      * 
      * </ul>
      * 
      * @param  out       The log to report the test outcome to; cannot be null.
      * @param  arguments Additional arguments, if any.
      * @return True if the tests succeeded, false if not.
      * @throws NullPointerException If {@code out} is null.
      * @throws Exception In case an unexpected error occurs.
      */
    public boolean test(Log out, Arguments arguments) throws Exception;
    
}
