/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Prototype</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.117]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Specify the kinds of objects to create using a prototypical 
  *    instance, and create new objects by copying this prototype.
  *    
  * </i></ul>
  *  
  * Here, the <b>Prototype</b> participant is represented by the
  * {@link dk.rode.thesis.prototype.Copyable} interface and any 
  * {@code Copyable} implementation is thus a representative 
  * of the <b>ConcretePrototype</b> participant. <p>   
  * 
  * The {@link dk.rode.thesis.meta.model.Sequence} interface extends 
  * {@code Copyable}. This effectively makes every {@code Sequence} 
  * implementation a representative of the <b>ConcretePrototype</b> 
  * participant as well. This package explicitly defines two concrete 
  * prototypical sequences, namely the 
  * {@link dk.rode.thesis.prototype.CountdownSequence} and 
  * {@link dk.rode.thesis.prototype.SymbolSequence} classes. <p>
  * 
  * Furthermore, the {@linkplain dk.rode.thesis.proxy Proxy} pattern is used to 
  * generate <i>dynamic proxies</i> via the 
  * {@link dk.rode.thesis.prototype.PrototypeFactory} class that makes 
  * any type (represented by an interface) copyable if the type implements 
  * {@code Copyable}, naturally, <b>and/or</b> if it supplies a 
  * {@link dk.rode.thesis.meta.reflect.Reflection#getCopyConstructor(Class)
  * copy} constructor. Hence, any such object also becomes a representation
  * of the <b>ConcretePrototype</b> pattern. <p> 
  * 
  * The <b>Client</b> participant is represented by the test application,
  * i.e. the {@link dk.rode.thesis.prototype.Main} class, which asks
  * the prototypes to copy themselves. The {@code Main} class also illustrates
  * how to make instances of the non-copyable 
  * {@link dk.rode.thesis.iterator.SequenceIterator} class dynamically copyable 
  * by using the {@code PrototypeFactory} class. <p>
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<p>
  * <img src="doc-files/Prototype.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:<br>
  * The {@code SymbolSequence} type also supports Java's built in 
  * <i>clone</i> functionality. Any type implementing the 
  * {@link java.lang.Cloneable java.lang.Cloneable} interface is also a 
  * representative of the <b>Prototype</b> participant. 
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Prototype", 
         scope        = Scope.Object,
         purpose      = Purpose.Creational,
         participants = {"Prototype", "ConcretePrototype", "Client"})
package dk.rode.thesis.prototype;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;

