/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.singleton;

import static dk.rode.thesis.meta.util.Const.SERIAL_VERSION_UID;
import dk.rode.thesis.meta.reflect.CallerClass;

/**
  * A <i>singleton error</i> is thrown because of illegal access 
  * to a singleton constructor. <p>
  * 
  * <b>Implementation notes</b>:<br>
  * Declared as an error because illegal access to a singleton constructor
  * is most definitely a programmer error.  
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    SimpsonsFamilySequence#SimpsonsFamilySequence(String...)
  * @see    SimpsonsFamilySequence#SimpsonsFamilySequence(SimpsonsFamilySequence)
  */
public class SingletonError extends Error {

    /**
      * Serial version id. 
      */
    final static long serialVersionUID = SERIAL_VERSION_UID;
    
    /**
      * No-arg constructor.
      */
    public SingletonError() {
        super();
    }

    /**
      * Constructor.
      * 
      * @param caller The illegal caller; can be null.
      */
    public SingletonError(CallerClass caller) {
        super(caller == null ? null : caller.toString());
    }

}
