/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.state;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;
import dk.rode.thesis.meta.model.Sequence.State;

/**
  * A <i>functional state</i> encapsulates specific state related 
  * functionality of a {@link StateableSequence stateable} sequence. <p>
  * 
  * Transitions between functional states are based on requests 
  * using <i>internal</i> {@link dk.rode.thesis.meta.model.Sequence.State 
  * states} via the {@link #action(Sequence.State, StateableSequence)} method.  
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @param  <E> The type of values delivered by applicable stateable sequences. 
  */
@Participant("State")
public interface FunctionalState<E> {

    /**
      * Depending on the value of the <i>internal</i> state supplied as 
      * {@code internalState}, the following forms of actions are 
      * inferred: <ul><p>
      * 
      * <li> {@link State#START}: initialisation requested. <p>
      * 
      * <li> {@link State#RESET}: reset requested. <p>
      * 
      * <li> {@link State#RESTART}: restart requested. <p>
      * 
      * <li> {@link State#NORMAL}: next value requested. <p>
      * 
      * <li> {@code null}: current value requested. <p>
      * 
      * </ul>
      * 
      * @param  internalState The internal state representing the requested action;
      *                       can be null (current).
      * @param  sequence The stateable sequence in question; cannot be null.               
      * @return The sequence result from the action taken; can be null.
      * @throws NullPointerException If {@code sequence} is null.
      */
    public E action(State internalState, StateableSequence<E> sequence);
        
}
