/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.templatemethod;

import static dk.rode.thesis.meta.util.Const.SERIAL_VERSION_UID;

import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>sequence value exception</i> is thrown by a  
  * sequence {@link SequenceTemplate template} in case of value 
  * related errors, such as initialisation error, a 
  * {@link SequenceTemplate#next() next} value cannot be delivered, or 
  * if the sequence is {@link SequenceTemplate#close() closed}. <p>
  * 
  * <b>Implementation notes</b>:<br> 
  * This exception type is declared as a runtime exception to maintain 
  * the semantics of the {@link Sequence} interface, e.g. sequence methods 
  * such as {@link Sequence#next()} cannot throw checked exceptions. 
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public class SequenceValueException extends RuntimeException {

    /**
      * Serial version id. 
      */
    final static long serialVersionUID = SERIAL_VERSION_UID;
    
    /**
      * No-arg constructor.
      */
    public SequenceValueException() {
        super();
    }

    /**
      * Constructor.
      * 
      * @param message The detail message; can be null.
      */
    public SequenceValueException(String message) {
        super(message);
    }

    /**
      * Constructor.
      * 
      * @param cause The cause of this exception; can be null.
      */
    public SequenceValueException(Throwable cause) {
        super(cause);
    }

    /**
      * Constructor.
      * 
      * @param message The detail message; can be null.
      * @param cause   The cause of this exception; can be null.
      */
    public SequenceValueException(String message, Throwable cause) {
        super(message, cause);
    }

}
